﻿using System;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using System.Windows.Forms;
using Wilco.SyntaxHighlighting;
using Wilco.Windows.SyntaxHighlighting;

namespace Kong.SyntaxHighlighter.Winform
{
    public partial class MainForm : Form
    {
        private readonly HtmlParser _htmlParser;
        private readonly RtfParser _parser;
        private HighlighterBase _highlighter;
        private bool _processSourceCode;

        public MainForm()
        {
            InitializeComponent();
            _highlighter = Register.Instance.Highlighters["C#"];
            _parser = new RtfParser(CodeTextBox);
            _htmlParser = new HtmlParser();
            _highlighter.Parser = _parser;
            LanguageComBox.SelectedIndex = 0;
        }

        private void richTextBox1_TextChanged(object sender, EventArgs e)
        {
            SetRichText(CodeTextBox.Text);
        }

        private void SetRichText(string text)
        {
            if (!_processSourceCode)
            {
                _processSourceCode = true;
                int start = CodeTextBox.SelectionStart,
                    length = CodeTextBox.SelectionLength;
                CodeTextBox.SuspendLayout();
                CodeTextBox.Rtf = _highlighter.Parse(text);
                CodeTextBox.SelectionStart = start;
                CodeTextBox.SelectionLength = length;
                _processSourceCode = false;
            }
        }

        private void LanguageComBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            var format = LanguageComBox.SelectedItem as string;
            if (string.IsNullOrEmpty(format)) return;

            _highlighter = Register.Instance.Highlighters[format];
            _highlighter.Parser = _parser;

            SetRichText(CodeTextBox.Text);
        }
        private string GetHtml(string content)
        {
            _highlighter.Parser = _htmlParser;
            string html = _highlighter.Parse(content);
            _highlighter.Parser = _parser;
            if (html != null)
            {
                html = html.Replace("\t", "   ");
                //清除首尾空行
                html = Regex.Replace(html, @"(^\s*\n)|(\n\s*$)", "", RegexOptions.Singleline); 
                
                //清除掉公共的空格
                MatchCollection matches = Regex.Matches(html, @"^ *(?=\S)", RegexOptions.Multiline);
                int len = matches.OfType<Match>().Select(m => m.Value.Length).Min();
                html = Regex.Replace(html, @"^ {" + len + "}", "", RegexOptions.Multiline);

                //把每一行开头的空格变成&nbsp;
                html = Regex.Replace(html, @"^ +(?=\S)",
                                     new MatchEvaluator(
                                         m => string.Join("", new string[m.Length].Select(s => "&nbsp;"))),
                                     RegexOptions.Multiline);
            }
            return html;
        }

        private string FormatCode(string code)
        {
            code = FormatLines(code);

            Settings settings = Settings.Default;
            return string.Format(settings.BoxFormat, settings.BorderColor.ToHexString(),
                                 settings.BackgroundColor.ToHexString(), code);
        }

        private string FormatLines(string code)
        {
            var settings = Settings.Default;
            var sb = new StringBuilder();
            string[] lines = code.Split('\n');
            for (int i = 0; i < lines.Length; ++i)
            {
                string line = lines[i];
                if (i % 2 == 0 || !settings.EnableAltLine)
                    line = String.Format(settings.LineFormat, settings.LineColor.ToHexString(), line);
                else
                    line = String.Format(settings.LineFormat, settings.AltLineColor.ToHexString(), line);
                sb.Append(line);
            }
            return sb.ToString();
        }

        public void CopyToClipboard()
        {
            string html = GetHtml(CodeTextBox.Text);
            html = FormatCode(html);
            HtmlFragment.CopyToClipboard(html);
        }

        private void InsertButton_Click(object sender, EventArgs e)
        {
            DialogResult = DialogResult.OK;
        }

        private void CancelButton_Click(object sender, EventArgs e)
        {
            DialogResult = DialogResult.Cancel;
        }
    }
}